// SPDX-License-Identifier: GPL-2.0
// Copyright (C) 2021 Jakub Horak
enum {
	LedWidth = 9,
};

static byte led_disp[LedWidth], led_disp_ptr, prev_led_disp_ptr;
static byte led_buf[LedWidth], led_ptr;

static const byte font[] = {
#include "7seg_font.h"
};

static byte
get_glyph(byte c) {
	if (c < 0x20 || c >= 0x80)
		return 0;
	return font[c - 0x20];
}

static inline void
set_led(byte n, byte val)
{
        if (n < 9)
                pin_write(PinLedSelect0 + n, val);
}

void
display_led_tick(void)
{
	byte d = led_disp[led_disp_ptr];
	for (byte i = 0; i < 8; i++) {
		pin_write(PinShiftDs, !!(d & 0x80));
		strobe(PinShiftSh, NORMAL);
		d <<= 1;
	}
        set_led(prev_led_disp_ptr, 1);
	strobe(PinShiftSt, NORMAL);
        set_led(led_disp_ptr, 0);
        prev_led_disp_ptr = led_disp_ptr++;
	if (led_disp_ptr >= LedWidth)
		led_disp_ptr = 0;
}

void
led_display_setup(void)
{
        for (byte i = 0; i < LedWidth; i++)
                pin_mode(PinLedSelect0 + i, OUTPUT);
	pin_mode(PinShiftDs, OUTPUT);
	pin_mode(PinShiftSt, OUTPUT);
	pin_mode(PinShiftSh, OUTPUT);
}


static void
xcls(void)
{
	memset(led_buf, 0, sizeof(led_buf));
        led_ptr = 0;
	printf("\e[1;1G");
}

static void
xputc(byte c)
{
	if (c == '.') {
		if (led_ptr > 0)
			led_ptr--;
	}
	if (led_ptr < LedWidth) {
		led_buf[led_ptr] |= get_glyph(c);
		led_ptr++;
	}
	putchar(c);
}

static void
xputs(const char *s)
{
	while (*s)
		xputc(*s++);
}

static void
xpresent(void)
{
	memcpy(led_disp, led_buf, LedWidth);
}
