// SPDX-License-Identifier: GPL-2.0
// Copyright (C) 2021 Jakub Horak

#include <util/atomic.h>

static volatile uint32_t world_clock_ms;

static uint32_t
millis(void)
{
	uint32_t world_clock_ms_copy;
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE) {
		world_clock_ms_copy = world_clock_ms;
	}
	return world_clock_ms_copy;
}

static inline void
tick_world_clock(void)
{
	world_clock_ms++;
}

#ifndef NO_TIMER
enum {
	TimerClock = 1,
};

static inline void
start_world_clock(void)
{
	timer_set_mode(TimerClock, TIMER_CTC, PRESCALER_64X);
	timer_output(TimerClock, TIMER_PIN_A, 249);
	timer_enable_irq(TimerClock, TIMER_IRQ_A);
}

ISR(TIMER1_COMPA_vect)
{
	tick_world_clock();	
}
#endif
