// SPDX-License-Identifier: GPL-2.0
// Copyright (C) 2021 Jakub Horak

enum {
	INPUT,
	OUTPUT,
	INPUT_PULLUP,
};

#define I2C_SDA_PIN	18
#define I2C_SCL_PIN	19

#ifdef LGT8F328P
ALWAYS_INLINE static inline byte
pin_mask_e(byte pin)
{
	if (pin == 0)
		return _BV(1);
	else 
		return _BV(3);
}
#endif

ALWAYS_INLINE static inline byte
pin_mask(byte pin)
{
	if (pin < 8)
		return _BV(pin);
	else if (pin < 8 + 6)
		return _BV(pin - 8);
	else if (pin < 8 + 6 + 6)
		return _BV(pin - (8 + 6));
#ifdef LGT8F328P
	else if (pin < 8 + 6 + 6 + 2)
		return pin_mask_e(pin - (8 + 6 + 6));
#endif
	else
		return 0;
}

ALWAYS_INLINE static inline volatile byte *
pin_out_reg(byte pin)
{
	if (pin < 8)
		return &PORTD;
	else if (pin < 8 + 6)
		return &PORTB;
	else if (pin < 8 + 6 + 6)
		return &PORTC;
#ifdef LGT8F328P
	else if (pin < 8 + 6 + 6 + 2)
		return &PORTE;
#endif
	else
		return &PORTD;
}

ALWAYS_INLINE static inline volatile byte *
pin_in_reg(byte pin)
{
	if (pin < 8)
		return &PIND;
	else if (pin < 8 + 6)
		return &PINB;
	else if (pin < 8 + 6 + 6)
		return &PINC;
#ifdef LGT8F328P
	else if (pin < 8 + 6 + 6 + 2)
		return &PINE;
#endif
	else
		return &PIND;
}


ALWAYS_INLINE static inline volatile byte *
pin_dir_reg(byte pin)
{
	if (pin < 8)
		return &DDRD;
	else if (pin < 8 + 6)
		return &DDRB;
	else if (pin < 8 + 6 + 6)
		return &DDRC;
#ifdef LGT8F328P
	else if (pin < 8 + 6 + 6 + 2)
		return &DDRE;
#endif
	else
		return &DDRD;
}


ALWAYS_INLINE static inline void
pin_write(byte pin, byte val)
{
	if (val)
		*pin_out_reg(pin) |= pin_mask(pin);
	else
		*pin_out_reg(pin) &= ~pin_mask(pin);
}

ALWAYS_INLINE static inline void
pin_toggle(byte pin)
{
	*pin_out_reg(pin) ^= pin_mask(pin);
}

ALWAYS_INLINE static inline byte
pin_read(byte pin)
{
	return (*pin_in_reg(pin)) & pin_mask(pin);
}

ALWAYS_INLINE static inline void
pin_mode(byte pin, byte mode)
{
	if (mode == OUTPUT) {
		*pin_dir_reg(pin) |= pin_mask(pin);
	} else if (mode == INPUT) {
		*pin_dir_reg(pin) &= ~pin_mask(pin);
		*pin_out_reg(pin) &= ~pin_mask(pin);
	} else if (mode == INPUT_PULLUP) {
		*pin_dir_reg(pin) &= ~pin_mask(pin);
		*pin_out_reg(pin) |= pin_mask(pin);
	}
}

enum {
	NORMAL,
	INVERTED,
};

ALWAYS_INLINE static inline void
strobe(byte pin, byte mode)
{
	if (mode == NORMAL) {
		pin_write(pin, 1);
		pin_write(pin, 0);
	} else {
		pin_write(pin, 0);
		pin_write(pin, 1);
	}
}

/* shift out most significiant byte first */
ALWAYS_INLINE static inline void
shift_out_msb(byte clock, byte data, byte value)
{
	for (byte i = 0; i < 8; i++) {
		pin_write(data, value & 0x80);
		strobe(clock, NORMAL);
		value <<= 1;
	}
}

ALWAYS_INLINE static inline byte
shift_in_msb(byte clock, byte data)
{
	byte x = 0;
	for (byte i = 0; i < 8; i++) {
		x = (x << 1) | !!pin_read(data);
		strobe(clock, NORMAL);
	}
	return x;
}
